@@ -17,7 +17,7 @@ debug_mode="$7"
 
 create_venv $task_id $datahub_version $plugins $tmp_dir
 
-if (datahub ingest run --help | grep report-to); then
+if (datahub ingest run --help | grep report-to > /dev/null); then
   echo "This version of datahub supports report-to functionality"
   rm -f "$report_file"
   report_option="--report-to ${report_file}"
