@@ -13,12 +13,14 @@
 # limitations under the License.
 
 from asyncio import tasks
+from datetime import datetime, timedelta
 import os
 import sys
 import subprocess
 import logging
 import asyncio
 from collections import deque
+from typing import Optional
 from acryl.executor.execution.sub_process_task_common import SubProcessTaskUtil
 from acryl.executor.execution.task import Task
 from acryl.executor.execution.task import TaskError
@@ -88,39 +90,57 @@ class SubProcessIngestionTask(Task):
         )
 
         # 4. Monitor and report progress.
-        running = True
+        most_recent_log_ts: Optional[datetime] = None
 
         async def _read_output_lines() -> None:
             while True:
                 assert ingest_process.stdout
                 line_bytes = await ingest_process.stdout.readline()
                 if not line_bytes:
-                    nonlocal running
-                    running = False
+                    logger.info(f'Got EOF from subprocess exec_id={exec_id} - stopping log monitor')
                     break
                 line = line_bytes.decode("utf-8")
+                
+                nonlocal most_recent_log_ts
+                most_recent_log_ts = datetime.now()
 
                 sys.stdout.write(line)
+                sys.stdout.flush()
                 stdout_lines.append(line)
 
                 await asyncio.sleep(0)
         
         async def _report_progress() -> None:
             while True:
-                if not running:
+                if ingest_process.returncode is not None:
+                    logger.info(f'Detected subprocess return code exec_id={exec_id} - stopping logs reporting')
                     break
 
                 await asyncio.sleep(self.config.heartbeat_time_seconds)
 
                 # Report progress
                 if ctx.request.progress_callback:
+                    if most_recent_log_ts is None:
+                        report = 'No logs yet'
+                    else:
+                        report = SubProcessTaskUtil._format_log_lines(stdout_lines)
+                        current_time = datetime.now()
+                        if most_recent_log_ts < current_time - timedelta(minutes=2):
+                            message = f'WARNING: These logs appear to be stale. No new logs have been received since {most_recent_log_ts} ({(current_time - most_recent_log_ts).seconds} seconds ago). ' \
+                                "However, the ingestion process still appears to be running and may complete normally."
+                            report = f'{report}\n\n{message}'
+
                     # TODO maybe use the normal report field here?
-                    ctx.request.progress_callback(f"stdout={SubProcessTaskUtil._format_log_lines(stdout_lines)}")
+                    ctx.request.progress_callback(report)
 
                 await asyncio.sleep(0)
         
+        async def _process_waiter() -> None:
+            await ingest_process.wait()
+            logger.info(f'Detected subprocess exited exec_id={exec_id}')
+        
         try: 
-            await tasks.gather(_read_output_lines(), _report_progress(), ingest_process.wait())
+            await tasks.gather(_read_output_lines(), _report_progress(), _process_waiter())
         except asyncio.CancelledError:
             # Terminate the running child process 
             ingest_process.terminate()
